@@ -1,3 +1,5 @@
+from typing import Optional
+
 from collections import namedtuple, defaultdict
 import copy
 import json
@@ -336,10 +338,9 @@ class ShardReader(object):
     minishard_index = np.copy(np.frombuffer(minishard_index, dtype=np.uint64))
     minishard_index = minishard_index.reshape( (3, len(minishard_index) // 3), order='C' ).T
 
-    for i in range(1, minishard_index.shape[0]):
-      minishard_index[i, 0] += minishard_index[i-1, 0]
-      minishard_index[i, 1] += minishard_index[i-1, 1] + minishard_index[i-1, 2]
-
+    minishard_index[:,0] = np.cumsum(minishard_index[:,0])
+    minishard_index[:,1] = np.cumsum(minishard_index[:,1])
+    minishard_index[1:,1] += np.cumsum(minishard_index[:-1,2])
     minishard_index[:,1] += self.spec.index_length()
 
     return minishard_index 
@@ -541,8 +542,9 @@ class ShardReader(object):
     return shattered
 
   def get_data(
-    self, label, path="", 
-    progress=None, parallel=1
+    self, label:int, path:str = "", 
+    progress:Optional[bool] = None, parallel:int = 1,
+    raw:bool = False
   ):
     """Fetches data from shards.
 
@@ -550,6 +552,7 @@ class ShardReader(object):
     path: subdirectory path
     progress: display progress bars
     parallel: (int >= 0) use multiple processes
+    raw: if true, don't decompress or decode stream
 
     Return: 
       if label is a scalar:
@@ -636,7 +639,7 @@ class ShardReader(object):
     del bundles
     del bundles_resp
 
-    if self.spec.data_encoding != 'raw':
+    if not raw and self.spec.data_encoding != 'raw':
       for filepath, binary in tqdm(binaries.items(), desc="Decompressing", disable=(not progress)):
         if binary is None:
           continue
